@@ -288,6 +288,49 @@ const ERROR_TESTS: {
   validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasForbiddenKeys([`foo`, `bar`])]),
   tests: [
     [{foo: 42}, [`.: Forbidden property "foo"`]],
+    [{foo: undefined, bar: null}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: false, bar: null}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: false, bar: 0}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: 42, bar: 42}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{baz: 42}, []],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasForbiddenKeys([`foo`, `bar`], { missingIf: 'missing' })]),
+  tests: [
+    [{foo: 42}, [`.: Forbidden property "foo"`]],
+    [{foo: undefined, bar: null}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: false, bar: null}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: false, bar: 0}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: 42, bar: 42}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{baz: 42}, []],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasForbiddenKeys([`foo`, `bar`], { missingIf: 'undefined' })]),
+  tests: [
+    [{foo: 42}, [`.: Forbidden property "foo"`]],
+    [{foo: undefined, bar: null}, [`.: Forbidden property "bar"`]],
+    [{foo: false, bar: null}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: false, bar: 0}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: 42, bar: 42}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{baz: 42}, []],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasForbiddenKeys([`foo`, `bar`], { missingIf: 'nil' })]),
+  tests: [
+    [{foo: 42}, [`.: Forbidden property "foo"`]],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, [`.: Forbidden property "foo"`]],
+    [{foo: false, bar: 0}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{foo: 42, bar: 42}, [`.: Forbidden properties "foo" and "bar"`]],
+    [{baz: 42}, []],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasForbiddenKeys([`foo`, `bar`], { missingIf: 'falsy' })]),
+  tests: [
+    [{foo: 42}, [`.: Forbidden property "foo"`]],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
     [{foo: 42, bar: 42}, [`.: Forbidden properties "foo" and "bar"`]],
     [{baz: 42}, []],
   ],
@@ -296,13 +339,152 @@ const ERROR_TESTS: {
   tests: [
     [{foo: 42}, [`.: Missing required property "bar"`]],
     [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
     [{baz: 42}, [`.: Missing required properties "foo" and "bar"`]],
   ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasRequiredKeys([`foo`, `bar`], { missingIf: 'missing' })]),
+  tests: [
+    [{foo: 42}, [`.: Missing required property "bar"`]],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
+    [{baz: 42}, [`.: Missing required properties "foo" and "bar"`]],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasRequiredKeys([`foo`, `bar`], { missingIf: 'undefined' })]),
+  tests: [
+    [{foo: 42}, [`.: Missing required property "bar"`]],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, [`.: Missing required property "foo"`]],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
+    [{baz: 42}, [`.: Missing required properties "foo" and "bar"`]],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasRequiredKeys([`foo`, `bar`], { missingIf: 'nil' })]),
+  tests: [
+    [{foo: 42}, [`.: Missing required property "bar"`]],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, [`.: Missing required properties "foo" and "bar"`]],
+    [{foo: false, bar: null}, [`.: Missing required property "bar"`]],
+    [{foo: false, bar: 0}, []],
+    [{baz: 42}, [`.: Missing required properties "foo" and "bar"`]],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasRequiredKeys([`foo`, `bar`], { missingIf: 'falsy' })]),
+  tests: [
+    [{foo: 42}, [`.: Missing required property "bar"`]],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, [`.: Missing required properties "foo" and "bar"`]],
+    [{foo: false, bar: null}, [`.: Missing required properties "foo" and "bar"`]],
+    [{foo: false, bar: 0}, [`.: Missing required properties "foo" and "bar"`]],
+    [{baz: 42}, [`.: Missing required properties "foo" and "bar"`]],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasAtLeastOneKey([`foo`, `bar`])]),
+  tests: [
+    [{foo: 42}, []],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
+    [{baz: 42}, [`.: Missing at least one property from "foo" or "bar"`]],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasAtLeastOneKey([`foo`, `bar`], { missingIf: 'missing' })]),
+  tests: [
+    [{foo: 42}, []],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
+    [{baz: 42}, [`.: Missing at least one property from "foo" or "bar"`]],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasAtLeastOneKey([`foo`, `bar`], { missingIf: 'undefined' })]),
+  tests: [
+    [{foo: 42}, []],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
+    [{baz: 42}, [`.: Missing at least one property from "foo" or "bar"`]],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasAtLeastOneKey([`foo`, `bar`], { missingIf: 'nil' })]),
+  tests: [
+    [{foo: 42}, []],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, [`.: Missing at least one property from "foo" or "bar"`]],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
+    [{baz: 42}, [`.: Missing at least one property from "foo" or "bar"`]],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasAtLeastOneKey([`foo`, `bar`], { missingIf: 'falsy' })]),
+  tests: [
+    [{foo: 42}, []],
+    [{foo: 42, bar: 42}, []],
+    [{foo: undefined, bar: null}, [`.: Missing at least one property from "foo" or "bar"`]],
+    [{foo: false, bar: null}, [`.: Missing at least one property from "foo" or "bar"`]],
+    [{foo: false, bar: 0}, [`.: Missing at least one property from "foo" or "bar"`]],
+    [{baz: 42}, [`.: Missing at least one property from "foo" or "bar"`]],
+  ],
 }, {
   validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasMutuallyExclusiveKeys([`foo`, `bar`])]),
   tests: [
     [{foo: 42}, []],
     [{bar: 42}, []],
+    [{foo: undefined, bar: null}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{foo: false, bar: null}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{foo: 42, bar: 42, baz: 42}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{baz: 42}, []],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasMutuallyExclusiveKeys([`foo`, `bar`], { missingIf: 'missing' })]),
+  tests: [
+    [{foo: 42}, []],
+    [{bar: 42}, []],
+    [{foo: undefined, bar: null}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{foo: false, bar: null}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{foo: false, bar: 0}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{foo: 42, bar: 42, baz: 42}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{baz: 42}, []],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasMutuallyExclusiveKeys([`foo`, `bar`], { missingIf: 'undefined' })]),
+  tests: [
+    [{foo: 42}, []],
+    [{bar: 42}, []],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{foo: false, bar: 0}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{foo: 42, bar: 42, baz: 42}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{baz: 42}, []],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasMutuallyExclusiveKeys([`foo`, `bar`], { missingIf: 'nil' })]),
+  tests: [
+    [{foo: 42}, []],
+    [{bar: 42}, []],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{foo: 42, bar: 42, baz: 42}, [`.: Mutually exclusive properties "foo" and "bar"`]],
+    [{baz: 42}, []],
+  ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasMutuallyExclusiveKeys([`foo`, `bar`], { missingIf: 'falsy' })]),
+  tests: [
+    [{foo: 42}, []],
+    [{bar: 42}, []],
+    [{foo: undefined, bar: null}, []],
+    [{foo: false, bar: null}, []],
+    [{foo: false, bar: 0}, []],
     [{foo: 42, bar: 42, baz: 42}, [`.: Mutually exclusive properties "foo" and "bar"`]],
     [{baz: 42}, []],
   ],
@@ -326,6 +508,17 @@ const ERROR_TESTS: {
     [{foo: 42, bar: false, baz: 42}, [`.: Property "foo" forbids using property "baz"`]],
     [{foo: 42, qux: 42}, []],
   ],
+}, {
+  validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasKeyRelationship(`foo`, t.KeyRelationship.Forbids, [`bar`, `baz`], {missingIf: `falsy`})]),
+  tests: [
+    [{foo: 42}, []],
+    [{bar: 42}, []],
+    [{foo: false, bar: 42}, []],
+    [{foo: 42, bar: false}, []],
+    [{foo: 42, bar: false, baz: false}, []],
+    [{foo: 42, bar: false, baz: 42}, [`.: Property "foo" forbids using property "baz"`]],
+    [{foo: 42, qux: 42}, []],
+  ],
 }, {
   validator: () => t.cascade(t.isRecord(t.isUnknown()), [t.hasKeyRelationship(`foo`, t.KeyRelationship.Requires, [`bar`, `baz`])]),
   tests: [
@@ -480,6 +673,15 @@ const COERCION_TESTS: {
     [[[`val`, 42]], [], {val: 42}],
     [[[`val`, 42, 12]], [`.[0]: Expected to have a length of exactly 2 elements (got 3)`]],
   ]
+}, {
+  validator: () => t.isPayload(t.isObject({foo: t.isBoolean()})),
+  tests: [
+    [`{"foo": true}`, [], {foo: true}],
+    [`{"foo": 1}`, [], {foo: true}],
+    [`{"foo": 0}`, [], {foo: false}],
+    [`{"foo": "true"}`, [], {foo: true}],
+    [`{"foo": "false"}`, [], {foo: false}],
+  ],
 }];
 
 describe(`Coercion Tests`, () => {
